/******************************************************
 * $Workfile::                                       
 * $Author::                                         
 * $Date::                                           
 * $Revision::                                       
 *
 *
 * This header files defines the input/output
******************************************************/
`include "../src/define.v"
//`include "../src/tb_define.v"

`timescale 1 ns / 10 ps

module spi_slave
  (
   CLK,
   RST_N,

  
   SPI_DATA_OUT, //To external SPI bus
   SPI_CLK, //From external SPI bus
   SPI_DATA_IN, //From external SPI bus
   SPI_CS_N,  //From external SPI bus
  
   TEST_SIGNAL_1,
   TEST_SIGNAL_2,
   TEST_SIGNAL_3,
   TEST_SIGNAL_4,

   TEST_SIGNAL_5,
   TEST_SIGNAL_6,
   TEST_SIGNAL_7,
   TEST_SIGNAL_8
   );



   //----------------------------------------------
   // Parameter Declarations                        
   //----------------------------------------------
   parameter IDLE                   = 0,
	     INC_BYTE_COUNT				= 1,
	     DELAY_CHECK_BYTE_COUNT		= 2,
	     CHECK_BYTE_COUNT			= 3,
	     RECEIVE_DATA				= 4,
		 WRITE_REGISTER 		    = 5,
	     STORE_VALUE         		= 6,
		 LOAD_SHIFT_REG				= 7,
		 DELAY_WRITE_DATA_BYTE      = 8,
	     WRITE_DATA_BYTE			= 9,
	     WRITE_DATA_COMPLETE		= 10;
		 
   //----------------------------------------------
   // Bit Count Parameter Declarations                        
   //----------------------------------------------
   parameter BIT_COUNT_IDLE         = 0,
			 BIT_COUNT_CLK_HI		= 1,
			 BIT_COUNT_CLK_LOW		= 2,
			 BIT_COUNT_INC_BYTE		= 3;
			 
   //----------------------------------------------
   // Store Register Parameter Declarations                        
   //----------------------------------------------
   parameter STORE_REG_IDLE         = 0,
             STORE_REG_CHECK_RW     = 1,
			 STORE_REG_VALUE		= 2,
			 STORE_REG_LATCH		= 3,
			 STORE_REG_LOAD_SHIFT	= 4,
			 STORE_REG_INC_BYTE     = 5;

   //--------------------------------------------------
   // IO Port Declarations                           
   //--------------------------------------------------   
   input                       CLK, RST_N;

   output                      SPI_DATA_OUT; //To external SPI bus
   input                       SPI_CLK; //From external SPI bus
   input                       SPI_DATA_IN;  //From external SPI bus
   input                       SPI_CS_N;  //From external SPI bus
   
   output wire                 TEST_SIGNAL_1;
   output wire                 TEST_SIGNAL_2;
   output wire                 TEST_SIGNAL_3;
   output wire                 TEST_SIGNAL_4;

   output wire                 TEST_SIGNAL_5;
   output wire                 TEST_SIGNAL_6;
   output wire                 TEST_SIGNAL_7;
   output wire                 TEST_SIGNAL_8;
   

   //------------------------------------------------
   // Wire and Register Declarations                
   //------------------------------------------------
   	reg [11:0]                 state, next;

   
   	reg                        SPI_DATA_OUT;

    // Bit Count Registers
	reg [3:0]                  in_bit_count;
	reg [3:0]                  out_bit_count;
	reg [7:0]                  byte_count;
	
	// SPI Shift Registers
	reg [7:0]                  shift_in_reg, shift_out_reg;
	reg                        read_spi_data_in_once_flag;
	reg                        write_spi_data_out_once_flag;
	reg [7:0]                  total_busy_msgs;

	reg                        write_spi_data_out_response_once;

	reg                        load_shift_reg_high_byte;

	reg                        spi_clk_reg;
	
	// Storage Registers
	reg [7:0]                  storage_register0;
	reg [7:0]                  storage_register1;
	reg [7:0]                  storage_register2;
	reg [7:0]                  storage_register3;
	reg [7:0]                  storage_register4;
	reg [7:0]                  storage_register5;
	reg [7:0]                  register_address; 
	reg [2:0]                  store_reg_state; 
	reg [7:0]                  register_value;
	
	
	// in bit counter
	reg [1:0]                  bit_count_state;
	
	//Test Signal
    reg [3:0]                  STATE_OUT;
	reg                        spi_test_reg;
   	
`ifdef SIM
   reg [8*26:1] state_name;
`endif

   //------------------------------------------------
   // Signal Assignments  
   //------------------------------------------------

   assign	TEST_SIGNAL_1 = STATE_OUT[0];
   assign	TEST_SIGNAL_2 = STATE_OUT[1];
   assign	TEST_SIGNAL_3 = STATE_OUT[2];
   assign	TEST_SIGNAL_4 = STATE_OUT[3];

   assign	TEST_SIGNAL_5 = store_reg_state[0];
   assign	TEST_SIGNAL_6 = store_reg_state[1];
   assign	TEST_SIGNAL_7 = bit_count_state[0];
   assign	TEST_SIGNAL_8 = bit_count_state[1];

   //-----------------------------------------------
   // Create a State Test Output Signal
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	  STATE_OUT <= 0;
	else
	  begin
	     if (state[IDLE])
           STATE_OUT <= 4'h0;
	     if (state[INC_BYTE_COUNT])
	       STATE_OUT <= 4'h1;
	     if (state[DELAY_CHECK_BYTE_COUNT])
	       STATE_OUT <= 4'h2;
	     if (state[CHECK_BYTE_COUNT])
	       STATE_OUT <= 4'h3;
	     if (state[RECEIVE_DATA])
	       STATE_OUT <= 4'h4;
	     if (state[WRITE_REGISTER])
	       STATE_OUT <= 4'h5;
	     if (state[STORE_VALUE])
	       STATE_OUT <= 4'h6;
	     if (state[LOAD_SHIFT_REG])
	       STATE_OUT <= 4'h7;
	     if (state[DELAY_WRITE_DATA_BYTE])
	       STATE_OUT <= 4'h8;
	     if (state[WRITE_DATA_BYTE])
	       STATE_OUT <= 4'h9;
	     if (state[WRITE_DATA_COMPLETE])
	       STATE_OUT <= 4'ha;
	  end // else: !if(!RST_N)
     end // always @ (posedge CLK or negedge RST_N)

	 //-----------------------------------------------
   // Create a spi test signal. 
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	 spi_test_reg <= 0;
	else
	  begin
	     if ( write_spi_data_out_response_once )
	       spi_test_reg <= 1;
	  end
     end // always @ (posedge CLK or negedge RST_N)


	 //-----------------------------------------------
   // Create a spi_clk_reg signal. 
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	 spi_clk_reg <= 0;
	else
	  begin
	     if ( SPI_CLK )
	       spi_clk_reg <= 1;
	     else 
	       spi_clk_reg <= 0;
	  end
     end // always @ (posedge CLK or negedge RST_N)


   //-----------------------------------------------
   // Register Block access
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
    begin
	if (!RST_N)
	begin
	  	storage_register0 <= 0;
	  	storage_register1 <= 0;
	  	storage_register2 <= 0;
	  	storage_register3 <= 0;
	  	storage_register4 <= 0;
	  	storage_register5 <= 0;
		register_address  <= 0;
		register_value    <= 0;
		store_reg_state <= STORE_REG_IDLE;
	end
	else
	  begin
 	     case( store_reg_state )
		     STORE_REG_IDLE:
			 begin
			     if ( state[WRITE_REGISTER] && !spi_clk_reg )
				 begin
				     register_address <= shift_in_reg;
			         //if( !register_address[7] ) 
				     //    store_reg_state <= STORE_REG_VALUE;
					 //else
				     //    store_reg_state <= STORE_REG_LOAD_SHIFT;
				     store_reg_state <= STORE_REG_CHECK_RW;
				 end
			 end
		     STORE_REG_CHECK_RW:
				 begin
			         if( !register_address[7] ) 
				         store_reg_state <= STORE_REG_VALUE;
					 else
				         store_reg_state <= STORE_REG_LOAD_SHIFT;
				 end
			 STORE_REG_VALUE:
			 begin
			     if( state[STORE_VALUE]  && !spi_clk_reg )
				 begin
				     store_reg_state <= STORE_REG_LATCH;
	                 register_value <= shift_in_reg;
				 end
			 end
			 STORE_REG_LATCH:
			 begin
				     case( register_address[6:0] )
					     7'h00: storage_register0 <= register_value;
						 7'h01: storage_register1 <= register_value;
						 7'h02: storage_register2 <= register_value;
					     7'h03: storage_register3 <= register_value;
						 7'h04: storage_register4 <= register_value;
						 7'h05: storage_register5 <= register_value;
					 endcase
				     store_reg_state <= STORE_REG_IDLE;
			 end
			 STORE_REG_LOAD_SHIFT:
			 begin
				     case( register_address[6:0] )
					     7'h00:  register_value <=  storage_register0;
						 7'h01:  register_value <=  storage_register1;
						 7'h02:  register_value <=  storage_register2;
					     7'h03:  register_value <=  storage_register3;
						 7'h04:  register_value <=  storage_register4;
						 7'h05:  register_value <=  storage_register5;
					 endcase
					 if( state[LOAD_SHIFT_REG] )
				         store_reg_state <= STORE_REG_IDLE;
		     end
			 default:
			 begin
				     store_reg_state <= STORE_REG_IDLE;
			 end
		 endcase
	  end
    end // always @ (posedge CLK or negedge RST_N)

   //------------------------------------------------
   // Input bit counter, maintains the count of serial
   // bits shifted in on SPI_MOSI
   //------------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
	  in_bit_count <= 0;
	  bit_count_state <= BIT_COUNT_IDLE;
	end
	else
	  begin
	  
	     case( bit_count_state )
		     BIT_COUNT_IDLE:
			     if ( state[RECEIVE_DATA] )
				     bit_count_state <= BIT_COUNT_CLK_HI;
			 BIT_COUNT_CLK_HI:
			     if( spi_clk_reg )
				 begin
				     bit_count_state <= BIT_COUNT_CLK_LOW;
	                 in_bit_count <= in_bit_count + 1;
				 end
			 BIT_COUNT_CLK_LOW:
			     if( !spi_clk_reg )
				     //bit_count_state <= BIT_COUNT_IDLE;
				     bit_count_state <= BIT_COUNT_CLK_HI;
				 else if( in_bit_count >= 8'h8)
				     bit_count_state <= BIT_COUNT_INC_BYTE;
			 BIT_COUNT_INC_BYTE:
			     if( state[CHECK_BYTE_COUNT] )
				 begin
				     bit_count_state <= BIT_COUNT_IDLE;
	                 in_bit_count <= 0;
				 end
			 default:
			 begin
				     bit_count_state <= BIT_COUNT_IDLE;
			 end
		 endcase
	  end
     end

   //------------------------------------------------
   // Busy Count for the number of times to
   // send a busy response when queried
   //------------------------------------------------
   reg [7:0] busy_count;
   
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	  busy_count <= 0;
	else
	  begin
	     if (state[WRITE_DATA_BYTE] && (out_bit_count >= 8))
	       busy_count <= busy_count + 1;
	     else if (state[LOAD_SHIFT_REG])
	       busy_count <= 0;
	  end
     end // always @ (posedge CLK or negedge RST_N)

   //------------------------------------------------
   // Byte Counter
   //------------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	  byte_count <= 0;
	else
	  begin
		if ( state[INC_BYTE_COUNT] )
				byte_count <= byte_count + 1;
		else if ( state[STORE_VALUE] || state[WRITE_DATA_COMPLETE])
	  			byte_count <= 8'h00;
	  end
     end // always @ (posedge CLK or negedge RST_N)   

   //------------------------------------------------
   // Create an output bit counter
   //------------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
      begin
	  out_bit_count <= 0;
        write_spi_data_out_once_flag <= 1;
	end
	else
	  begin
	     if ( state[WRITE_DATA_BYTE] && !spi_clk_reg && write_spi_data_out_once_flag )
	     begin
	       	out_bit_count <= out_bit_count + 1;
			write_spi_data_out_once_flag <= 0; 
	     end
		else if( state[WRITE_DATA_BYTE] && spi_clk_reg && !write_spi_data_out_once_flag )
 		begin
        		write_spi_data_out_once_flag <= 1;
		end
	     //else if ( state[INC_BYTE_COUNT] )
	     else if ( state[CHECK_BYTE_COUNT] )
	       out_bit_count <= 0;
	  end
     end // always @ (posedge CLK or negedge RST_N)

   //------------------------------------------------
   // SPI input shift register
   //------------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
			shift_in_reg <= 0;
			read_spi_data_in_once_flag <= 1;
	end
	else
	  begin
	     //if ( state[RECEIVE_DATA] && !SPI_CLK && read_spi_data_in_once_flag )
	     if ( state[RECEIVE_DATA] && spi_clk_reg && read_spi_data_in_once_flag )
	       begin
		  shift_in_reg[0] <= SPI_DATA_IN;
		  shift_in_reg[7:1] <= shift_in_reg[6:0];
		  read_spi_data_in_once_flag <= 0;
	       end
		//else if ( state[RECEIVE_DATA] && SPI_CLK && !read_spi_data_in_once_flag )
		else if ( state[RECEIVE_DATA] && !spi_clk_reg && !read_spi_data_in_once_flag )
		begin
			read_spi_data_in_once_flag <= 1;
		end
		else if ( state[WRITE_REGISTER] || state[STORE_VALUE])
			read_spi_data_in_once_flag <= 1;
	     else if (state[IDLE])
	       shift_in_reg <= 0;
	  end
     end // always @ (posedge CLK or negedge RST_N)
 
   //------------------------------------------------
   // SPI Shift Data Response out
   //------------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	  begin
	     shift_out_reg <= 8'h30;
	     SPI_DATA_OUT <= 1;
		 total_busy_msgs <= 8'h01;
		 load_shift_reg_high_byte <= 0;
		 write_spi_data_out_response_once <= 0;
	  end
	else
	  begin
/*	  
	     if ( state[CHECK_BYTE_COUNT] )
	     begin
		  	if( load_shift_reg_high_byte )
			begin
		  		SPI_DATA_OUT <= 0;
				shift_out_reg <= register_value;
				load_shift_reg_high_byte <= !load_shift_reg_high_byte;
			end
		  	else if( !load_shift_reg_high_byte )
			begin
		  		SPI_DATA_OUT <= 0;
				shift_out_reg <= register_value;
				load_shift_reg_high_byte <= !load_shift_reg_high_byte;
			end
	     end
	     if (state[LOAD_SHIFT_REG] )
	     begin
		  		SPI_DATA_OUT <= 0;
	     end
*/
	     if (state[LOAD_SHIFT_REG] )
	     begin
	  		SPI_DATA_OUT <= 0;
			shift_out_reg <= register_value;
	     end
// Added new shift_out_reg <= register_value code
	     if (state[RECEIVE_DATA])
	     begin
		     SPI_DATA_OUT <= SPI_DATA_OUT;
		     shift_out_reg <= 0;
	     end
	     if (state[IDLE] )
	     begin
	  		SPI_DATA_OUT <= 0;
	     end
	     //if ( state[WRITE_REGISTER] || state[WRITE_DATA_BYTE] || state[INC_BYTE_COUNT])
	     if ( state[WRITE_DATA_BYTE] )
	       begin

			//if( !SPI_CLK  && write_spi_data_out_response_once )
			if( spi_clk_reg  && write_spi_data_out_response_once )
			begin
		  		SPI_DATA_OUT <= shift_out_reg[7];
		  		shift_out_reg[7:1] <= shift_out_reg[6:0];
		 		shift_out_reg[0] <= 0;
				write_spi_data_out_response_once <= 0;
			end
			//else if( SPI_CLK  && !write_spi_data_out_response_once )
			else if( !spi_clk_reg  && !write_spi_data_out_response_once )
			begin
				write_spi_data_out_response_once <= 1;
			end

	       end // if ((state[WRITE_BUSY_RESPONSE] || state[WRITE_CMD_RESPONSE]) &&...
	  end
     end // always @ (posedge CLK or negedge RST_N)   
   
	  
   //--------------------------------------------------
   // SPI State Machine
   //--------------------------------------------------  
   // Next State Logic
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	  begin
	     state <= 12'h00;
	     state[IDLE] <= 1'b1;
	  end
	else
	  state <= next;
     end

   always @ ( /*AUTOSENSE*/SPI_CLK or SPI_DATA_IN or busy_count or byte_count
	     or in_bit_count or out_bit_count or state  
		 or spi_clk_reg or store_reg_state or bit_count_state)
     begin
	next = 12'h00;

	if (state[IDLE])
	begin
        if ( spi_clk_reg )
			next[RECEIVE_DATA] = 1'b1;
		else
			next[IDLE] = 1'b1;
	end


	if (state[INC_BYTE_COUNT])
	 // next[DELAY_CHECK_BYTE_COUNT] = 1'b1;//Check instruction 
	  next[CHECK_BYTE_COUNT] = 1'b1;//Check instruction 

	//if (state[DELAY_CHECK_BYTE_COUNT])
	//  next[CHECK_BYTE_COUNT] = 1'b1;//Check instruction 

	if (state[CHECK_BYTE_COUNT])//Check instruction 
	  begin
		case( byte_count )
		 8'h01 :
			next[WRITE_REGISTER] = 1'b1;
		 8'h02 :
			next[STORE_VALUE] = 1'b1;
         8'h03 :
			next[IDLE] = 1'b1;
		endcase
	  end // if (state[CHECK_BYTE_COUNT])	     

	if (state[RECEIVE_DATA])
	  begin
	     if ((in_bit_count >= 8) && spi_clk_reg)
	       next[INC_BYTE_COUNT] = 1'b1;
	     else
	       next[RECEIVE_DATA] = 1'b1;
	  end

	if (state[WRITE_REGISTER])
	  begin
	     if ( store_reg_state == STORE_REG_LOAD_SHIFT)
	       next[LOAD_SHIFT_REG] = 1'b1;
	     else if ( store_reg_state == STORE_REG_VALUE )
	       next[IDLE] = 1'b1;
	     else 
	       next[WRITE_REGISTER] = 1'b1;
	  end

	  if (state[STORE_VALUE])
	  begin
	     if ( store_reg_state == STORE_REG_LATCH )
	       next[IDLE] = 1'b1;
	     else
	       next[STORE_VALUE] = 1'b1;
	  end

	if (state[LOAD_SHIFT_REG])
	       next[DELAY_WRITE_DATA_BYTE] = 1'b1;

	if (state[DELAY_WRITE_DATA_BYTE])
	  begin
	     //if ( SPI_DATA_IN )
	       next[WRITE_DATA_BYTE] = 1'b1;
	     //else
	     //  next[DELAY_WRITE_DATA_BYTE] = 1'b1;
	  end

	if (state[WRITE_DATA_BYTE])
	  begin
	     if (( out_bit_count >= 8 ) && spi_clk_reg )
	       next[WRITE_DATA_COMPLETE] = 1'b1;
	     else
	       next[WRITE_DATA_BYTE] = 1'b1;
	  end

	if (state[WRITE_DATA_COMPLETE])
	  begin
	     if ( !spi_clk_reg )
	       next[IDLE] = 1'b1;
	     else
	       next[WRITE_DATA_COMPLETE] = 1'b1;
	  end

	
`ifdef SIM
	if (state == (1 << IDLE))
	  state_name = "IDLE";
	else if (state == (1 << INC_BYTE_COUNT))
	  state_name = "INC_BYTE_COUNT";
	else if (state == (1 << DELAY_CHECK_BYTE_COUNT))
	  state_name = "DELAY_CHECK_BYTE_COUNT";
	else if (state == (1 << CHECK_BYTE_COUNT))
	  state_name = "CHECK_BYTE_COUNT";
	else if (state == (1 << RECEIVE_DATA))
	  state_name = "RECEIVE_DATA";
	else if (state == (1 << STORE_VALUE))
	  state_name = "STORE_VALUE";
	else if (state == (1 << WRITE_REGISTER))
	  state_name = "WRITE_REGISTER";
	else if (state == (1 << LOAD_SHIFT_REG))
	  state_name = "LOAD_SHIFT_REG";
	else if (state == (1 << DELAY_WRITE_DATA_BYTE))
	  state_name = "DELAY_WRITE_DATA_BYTE";
	else if (state == (1 << WRITE_DATA_BYTE))
	  state_name = "WRITE_DATA_BYTE";
	else if (state == (1 << WRITE_DATA_COMPLETE))
	  state_name = "WRITE_DATA_COMPLETE";
`endif
     end // always @ (...
   

endmodule // spi_slave

